<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}



/**
 * Woocommerce Address Autocomplete
**/
function gdprmap_include_autocomplete_js() {

    if ( class_exists( 'woocommerce' ) ) {

        $options = get_option( 'gdprmapwoo_options' );

        // check if options is activated and the cart page is loaded
        if( is_checkout() && $options['api_woocommerce'] == 'true' ) {
            
            wp_enqueue_script('gdpr-map-woo-autocomplete', plugin_dir_url( __DIR__ ) . 'js/address-autocomplete.js', array('jquery'));
            wp_localize_script('gdpr-map-woo-autocomplete', 'GDPRMAPWOO', array('api_key' => $options['api_key'], 'shippingactive' => $options['shippingactive'], 'billingactive' => $options['billingactive'], 'checkhouseno' => $options['checkhouseno'], 'maxresults' => $options['maxresults'], 'apirequests' => [0,0], 'housenowarning' => __('Please add a housenumber.', 'gdprmapwoo' ) ) );
            wp_enqueue_style('gdpr-map-front', plugin_dir_url( __DIR__ ) . 'css/style.css');
        }

    }

}
add_action('wp_enqueue_scripts', 'gdprmap_include_autocomplete_js');


/**
 *  Custom checkout validation hook, shipping / Billing address contains number.
 */
add_action('woocommerce_after_checkout_validation', function($data, $errors) {

    // disable validation if not active
    $options = get_option( 'gdprmapwoo_options' );
    if( $options['checkhouseno'] != 'true' ) {
        return;
    }

    // Function to check if a string contains at least one number
    function contains_number($string) {
        return preg_match('/\d/', $string);
    }

    // Get billing and shipping address from checkout fields
    $billing_address = isset($data['billing_address_1']) ? $data['billing_address_1'] : '';
    $shipping_address = isset($data['shipping_address_1']) ? $data['shipping_address_1'] : '';

    // Validate billing address
    if (!contains_number($billing_address)) {
        $errors->add('validation', __('Billing address must contain at least one number.', 'gdprmapwoo'));
    }

    // Validate shipping address (only if shipping is required)
    if (!empty($shipping_address) && !contains_number($shipping_address)) {
        //$errors->add('validation', __('Shipping address must contain at least one number.', 'gdprmapwoo'));
    }
}, 10, 2);