/**
 * GDPR Map WooCommerce Autocomplete
 */

jQuery(document).ready(function () {

	// Get enabled inputs
	var enabled_inputs = [];
	if( GDPRMAPWOO.billingactive == 'true' ) {
		enabled_inputs.push('#billing_address_1');
	}
	if( GDPRMAPWOO.shippingactive == 'true' ) {
		enabled_inputs.push('#shipping_address_1');
	}

	const input_ids = enabled_inputs.join(', ');

 
	// Init
	jQuery(input_ids).after('<div class="gdprmap-woocommerce-autocomplete-wrapper"></div>');

	// Input listener
	jQuery(input_ids).on( 'keyup click', function(event) {

		// space, arrows, numbers[0-9] -> no request
		const magic_keycodes = [32, 37, 38, 39, 40];
		if( event.type == 'keyup' && magic_keycodes.includes(event.originalEvent.keyCode) ) {
			return true;
		}		

		gdprmapwoo_address_autocomplete('#'+jQuery(this).attr('id'));

	});


	// remove on blur
	jQuery(input_ids).on( 'blur', function(){

		setTimeout( function(input){
			jQuery('.gdprmap-woocommerce-autocomplete-wrapper').html('');
			gdprmapwoo_check_housenumber(input)

		}, 300, '#'+jQuery(this).attr('id') );
	});
	

});



/**
 * Fuzzy search
 */
function gdprmapwoo_address_autocomplete( input ) {

	const input_prefix = input.split('_')[0];

	var search = jQuery(input).val();

	if( search.length < 3 ) {
		jQuery(input).next('.gdprmap-woocommerce-autocomplete-wrapper').html('');
		return false;
	}


	// abort pending requests
	if( GDPRMAPWOO.apirequests[0] != 0 ) {
		GDPRMAPWOO.apirequests[0].abort();
	}

	if( GDPRMAPWOO.apirequests[1] != 0 ) {
		GDPRMAPWOO.apirequests[1].abort();
	}


	// parse fuzzy street name
	var fuzzysearch = gdprmapwoo_parse_street_name(search);

	if( fuzzysearch.multisearch == 1 ) {

		// multi call
		GDPRMAPWOO.apirequests[0] = gdprmapwoo_api_call(search, input_prefix);
		GDPRMAPWOO.apirequests[1] = gdprmapwoo_api_call(fuzzysearch.variations[0].join(' '), input_prefix);

		jQuery.when( GDPRMAPWOO.apirequests[0], GDPRMAPWOO.apirequests[1] ).done(function(response1, response2) {

			gdprmapwoo_create_results_table(input, gdprmapwoo_parse_results(response1[0], input_prefix).concat(gdprmapwoo_parse_results(response2[0], input_prefix)));
		});

	}else{

		// single call
		GDPRMAPWOO.apirequests[0] = gdprmapwoo_api_call(search, input_prefix);

		jQuery.when( GDPRMAPWOO.apirequests[0] ).done(function(response1) {

			gdprmapwoo_create_results_table(input, gdprmapwoo_parse_results(response1, input_prefix));

		});
	}


}


/**
 * parse API results
 */
function gdprmapwoo_parse_results(data, input_prefix) {

	// values
	var country_top = jQuery(input_prefix+'_country').val();

	// deduplication array
	var dedup = [];
	var results = [];

	jQuery.each(data.features, function(k,v){

			// Check if wrong counry code
			if( typeof v.properties.countrycode != 'undefined' ) {
				if( country_top != v.properties.countrycode ) {
					return true;
				}
			}

			// check for osm_key highway
			var searchlist;
			if( typeof v.properties.osm_key != 'undefined' ) {
				searchlist = ['highway', 'building', 'place']; 
				if( !searchlist.includes(v.properties.osm_key) ) {
					return true; 
				}
				searchlist = ['apartments', 'bungalow', 'residential', 'semidetached_house', 'living_street', 'footway', 'house', 'path' ];
				if( typeof v.properties.osm_value != 'undefined' ) {
					if( v.properties.osm_key != 'highway' && !searchlist.includes(v.properties.osm_value) ) {
						return true;
					}
				}
			}

			// check for osm_value industrial
			if( typeof v.properties.osm_value != 'undefined' ) {
				searchlist = ['industrial', 'office', 'service', 'path'];
				if( searchlist.includes(v.properties.osm_value) ) {
					return true;
				}
			}

			var state = '';
			if( typeof v.properties.state != 'undefined' ) {
				state = v.properties.state.trim();
			}

			var city = '';
			if( typeof v.properties.city != 'undefined' ) {
				city = ' '+v.properties.city+' ';
			}

			var country = '';
			if( typeof v.properties.country != 'undefined' ) {
				country = ' '+v.properties.country+' ';
			}

			var name_arr = [];
			if( typeof v.properties.name != 'undefined' ) {
				name_arr.push(v.properties.name);
			}

			if( typeof v.properties.street != 'undefined' ) {
				name_arr.push(v.properties.street);
			}

			if( typeof v.properties.housenumber != 'undefined' ) {
				name_arr.push(v.properties.housenumber)
			}

			var postcode = '';
			if( typeof v.properties.postcode != 'undefined' ) {
				postcode = ' '+v.properties.postcode+' ';
			}
			
			var name = name_arr.join(' ');

			if( postcode != '' || city != '' ) {

				if( !dedup.includes(name+postcode+city) ) {
					results.push('<div class="gdprmap-woocommerce-autocomplete-item" data-city="'+city.trim()+'" data-street="'+name+'" data-zip="'+postcode.trim()+'" data-state="'+state+'"><span>'+name+', '+postcode+city+'</span></div>');
					dedup.push(name+postcode+city);
				}

			}else{

				if( !dedup.includes(name) ) {
					results.push('<div class="gdprmap-woocommerce-autocomplete-item" data-city="'+city.trim()+'" data-street="'+name+'" data-zip="'+postcode.trim()+'" data-state="'+state+'"><span>'+name+'</span></div>');
					dedup.push(name);
				}
			}
			
		});

	return results;
}



/** 
 * API call
 */
function gdprmapwoo_api_call(search, input_prefix) {

	// dropdown value
	//var prefilter = [];
	var country_top_text = jQuery(input_prefix+'_country option:selected').text();
	//

	/*
	prefilter.push(jQuery(input_prefix+'_country option:selected').text());

	var city = jQuery(input_prefix+'_city').val();
	if( city.trim() != "" ) {
		prefilter.push(city);
	}
	*/

	// Ajax Call
	var request = jQuery.ajax({
	  url: "https://api.gdpr-map.eu/api?q="+search+", "+country_top_text+"&layer=street&lang=de&layer=house&limit=30&key="+GDPRMAPWOO.api_key,
	  dataType: 'json'
	});


	// Attach an abort method to the function
  request.abortRequest = function() {
      request.abort();
  };

  return request;
}


/**
 * create results table
 */
function gdprmapwoo_create_results_table(input, results) {

	// prefix
	const input_prefix = input.split('_')[0];


	// table
	var table = '<div class="gdprmap-woocommerce-autocomplete">';

	results_limited = results.slice(0, GDPRMAPWOO.maxresults);

	var results_printed = 0;

	//results
	results_limited.forEach( (element) => {

		// Postfilter if zip code is done
		var post_filter_zip = jQuery(input_prefix+'_city').val();
		if( post_filter_zip.trim() != '' ) {

			if( element.includes(post_filter_zip.trim()) ) {
				table += element;
				results_printed++;
			}			

		}else{
 
			table += element;
			results_printed++;
		}		
		
	});


	// copyright
	table += '<div id="gdprmap-woocommerce-autocomplete-copyright"><span>Powered by <a href="https://www.gdpr-map.eu/" target="_blank">GDPRmap</a></span><span> | © <a href="https://www.openstreetmap.org/copyright" target="_blank">OpenStreetMap</a> contributors</span></div>'

	// table end
	table += '</div>';

	if( results_printed == 0 ) {
		table = '';
	}

		

	// append table to empty wrapper
	jQuery(input).next('.gdprmap-woocommerce-autocomplete-wrapper').html('');
	jQuery(input).next('.gdprmap-woocommerce-autocomplete-wrapper').append(table);

	// click listener for current suggestions
	jQuery(input).next('.gdprmap-woocommerce-autocomplete-wrapper').find('.gdprmap-woocommerce-autocomplete-item').click(function(){

		// populate fields
		if( typeof jQuery(this).attr('data-street') != 'undefined' ) {
			jQuery(input_prefix+'_address_1').val(jQuery(this).attr('data-street'));
		}
		
		if( typeof jQuery(this).attr('data-zip') != 'undefined' ) {
			jQuery(input_prefix+'_postcode').val(jQuery(this).attr('data-zip'));
			jQuery(input_prefix+'_postcode').removeAttr('aria-invalid');
			jQuery(input_prefix+'_postcode').removeAttr('aria-describedby');
			jQuery(input_prefix+'_postcode').remove(input_prefix+'_postcode'+'_description');

			jQuery(input_prefix+'_postcode').parent().parent().removeClass('woocommerce-invalid-required-field');
			jQuery(input_prefix+'_postcode').parent().parent().removeClass('woocommerce-invalid');
			jQuery(input_prefix+'_postcode').parent().parent().addClass('woocommerce-validated');
		}

		if( typeof jQuery(this).attr('data-city') != 'undefined' ) {
			jQuery(input_prefix+'_city').val(jQuery(this).attr('data-city'));
			jQuery(input_prefix+'_city').removeAttr('aria-invalid');
			jQuery(input_prefix+'_city').removeAttr('aria-describedby');
			jQuery(input_prefix+'_city').remove(input_prefix+'_city'+'_description');

			jQuery(input_prefix+'_city').parent().parent().removeClass('woocommerce-invalid-required-field');
			jQuery(input_prefix+'_city').parent().parent().removeClass('woocommerce-invalid');
			jQuery(input_prefix+'_city').parent().parent().addClass('woocommerce-validated');
		}


		// change state
		if( jQuery(this).attr('data-state') != '') {
			let select = jQuery(input_prefix+'_state'); // Change to your select element ID
			let value = select.find("option:contains('"+jQuery(this).attr('data-state')+"')").val();
			jQuery(input_prefix+'_state').val(value);
			jQuery(input_prefix+'_state').change();
		}
		    
		// reset wrapper
		jQuery(input).next('.gdprmap-woocommerce-autocomplete-wrapper').html('');
		gdprmapwoo_check_housenumber(input);
	});
}


/**
 * Street name parser
 */
function gdprmapwoo_parse_street_name(input) {
  // Normalize input to lowercase for easier pattern matching
  const normalizedInput = input.toLowerCase().trim();
  let result = [];
  let multisearch = 0;
  
  // Pattern 1: Detect concatenated street names (Balanstr, Balanstra, Balanstras, etc.)
  const streetSuffixPattern = /^(.*?)(?<!\s)(ers|st|str|stra|stras|strass|straß|straße|strasse)$/i;
  const match = normalizedInput.match(streetSuffixPattern);
  
  if (match) {
    // Found a potential concatenated street name 
    const [_, baseName, suffix] = match;
    
    // Only process if base name has at least 3 characters to avoid false positives
    if (baseName.length >= 3) {
      // Split the concatenated name [base, suffix]
      result.push([baseName, suffix]);
      multisearch = 1; 
    }
  }
  
  // Pattern 2: Detect separated street names (Balan Str, Balan Stra, etc.)
  const separatedPattern = /^(.*?)\s+(st|str|stra|stras|strass|straß|straße|strasse)$/i;
  const separatedMatch = normalizedInput.match(separatedPattern);
  
  if (separatedMatch) {
    // Found a potential separated street name
    const [_, baseName, suffix] = separatedMatch;
    
    // Only process if base name has at least 3 characters
    if (baseName.length >= 3) {
      // Combine the separated parts [baseName + suffix]
      result.push([baseName + suffix]);
      multisearch = 1;
    }
  }
  
  // If no patterns matched, return the original input
  if (result.length === 0) {
    result.push([normalizedInput]);
  }
  
  return {
    variations: result,
    multisearch
  };
}


/**
 * Check for housenumber
 */
function gdprmapwoo_check_housenumber(input) {

	// check active
	if( GDPRMAPWOO.checkhouseno != 'true' ) {
		return true;
	}

	// Housenumber warning
	var hasNumber = /\d/.test(jQuery(input).val());
	if(!hasNumber) {

		// no input
		if( jQuery(input).val().length <= 0 ) {
			return true;
		}

		// description existing
		if( jQuery(input+'_description').length > 0 ) {
			return true;
		}

		jQuery(input).attr('aria-invalid', 'true');
		jQuery(input).attr('aria-describedby', input.replace('#', '')+'_description');
		jQuery(input).parent().parent().append('<p id="'+input.replace('#', '')+'_description" class="checkout-inline-error-message">'+GDPRMAPWOO.housenowarning+'</p>');

		jQuery(input).parent().parent().addClass('woocommerce-invalid-required-field');
		jQuery(input).parent().parent().addClass('woocommerce-invalid');
		jQuery(input).parent().parent().removeClass('woocommerce-validated');
		

	}else{

		jQuery(input).removeAttr('aria-invalid');
		jQuery(input).removeAttr('aria-describedby');
		jQuery(input).remove(input+'_description');

		jQuery(input).parent().parent().removeClass('woocommerce-invalid-required-field');
		jQuery(input).parent().parent().removeClass('woocommerce-invalid');
		jQuery(input).parent().parent().addClass('woocommerce-validated');
	}
}